
class Simulation(numServers: Int, val meanServiceTime: Int,
		 val meanInterarrivalTime: Int, 
		 val coffeeServerFreq: Int,
		 val demandingClientFreq: Int) {
  var totalClients = 0
  val servers = new Array[Server](numServers)
  val queue: Queue[Client] = 
    new scala.collection.mutable.Queue[Client] with Queue[Client]
  var nextArrival = ExponentialRandom.randomInt(meanInterarrivalTime)

  for (j <- 0 until servers.length) {
    if (coffeeServerFreq != 0 && ((j + 1) % coffeeServerFreq) == 0)
      servers(j) = new CoffeeDrinkingServer(j)
    else
      servers(j) = new SimServer(j)
  }
  
  println("     Number of servers = " + servers.length); 
  println("     Mean service time = " + meanServiceTime); 
  println("Mean interarrival time = " + meanInterarrivalTime); 

  def checkClientArrival(t: Int) {
    if (t == nextArrival) {
      val client = 
	if (demandingClientFreq != 0 && 
	    (totalClients % demandingClientFreq) == 0)
	  new DemandingClient(totalClients, t, meanServiceTime)
	else
	  new SimClient(totalClients, t, meanServiceTime)
      queue.enqueue(client)
      totalClients += 1
      nextArrival = t + ExponentialRandom.randomInt(meanInterarrivalTime)
    }
  }

  def simulate(t: Int) {
    println("Time " + t + ": ")
    checkClientArrival(t)
    for (j <- 0 until servers.length) {
      val s = servers(j)
      if (s.client != null && t == s.client.stopTime)
	s.stopServing(t)
      if (s.isIdle(t) && !queue.isEmpty) {
	val client = queue.dequeue()
	s.startServing(client, t)
      }
    }
  }
}

object Simulation {
  def main(args: Array[String]) {
    if (args.length < 4 || args.length > 6) {
      println("Usage: scala Simulation <numServers> "
	      + "<timeSteps> <meanServiceTime> " 
	      + "<meanInterarrivalTime> " 
	      + "<optional coffeeserver frequency> "
	      + "<optional demanding client frequency>")
      println(" e.g.: scala Simulation 3 100 12 4")
      println("   or: scala Simulation 3 100 12 4 2 5")
      sys.exit(0)
    }

    val numServers = args(0).toInt
    val timeSteps = args(1).toInt
    val meanServiceTime = args(2).toInt
    val meanInterarrivalTime = args(3).toInt
    val coffeeServerFreq = if (args.length >= 5) args(4).toInt else 0
    val demandingClientFreq = if (args.length >= 6) args(5).toInt else 0
    
    val sim = new Simulation(numServers, meanServiceTime, meanInterarrivalTime,
			     coffeeServerFreq, demandingClientFreq)

    for (t <- 0 until timeSteps)
      sim.simulate(t)
  } 
}
